#include "SDL.h"
#include "SDL_events.h"
#include "SDL_image.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"
#include "SDL_thread.h"
#include "stdlib.h"
#include "stdio.h"
#include "time.h"
#include "string.h"

/* Display */
#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240
#define SCREEN_DEPTH 8

/* Text */
#define RENDER_MODE 2

/* Keys */
#define GP2X_BUTTON_UP              (0)
#define GP2X_BUTTON_DOWN            (4)
#define GP2X_BUTTON_LEFT            (2)
#define GP2X_BUTTON_RIGHT           (6)
#define GP2X_BUTTON_UPLEFT          (1)
#define GP2X_BUTTON_UPRIGHT         (7)
#define GP2X_BUTTON_DOWNLEFT        (3)
#define GP2X_BUTTON_DOWNRIGHT       (5)
#define GP2X_BUTTON_CLICK           (18)
#define GP2X_BUTTON_A               (12)
#define GP2X_BUTTON_B               (13)
#define GP2X_BUTTON_X               (14)
#define GP2X_BUTTON_Y               (15)
#define GP2X_BUTTON_L               (10)
#define GP2X_BUTTON_R               (11)
#define GP2X_BUTTON_START           (8)
#define GP2X_BUTTON_SELECT          (9)
#define GP2X_BUTTON_VOL_UP          (16)
#define GP2X_BUTTON_VOL_DOWN        (17)

void play_music(int soundtrack) {
      Mix_Music *music;
      Mix_HaltMusic();
      switch(soundtrack)
      {
          case 1:
               music = Mix_LoadMUS("audio/background_music.wav");
               Mix_PlayMusic(music, -1); 
               break;
          case 2:
               music = Mix_LoadMUS("audio/gameover_music.wav");
               Mix_PlayMusic(music, 1); 
               break;
      }
}

void show_button(SDL_Surface *screen, int random) {
      SDL_Rect    buttonarea;
      SDL_Surface *buttonpic;

      switch(random)
      {
          case 1:
               buttonpic = IMG_Load("img/button_up.png");
               break;
          case 2:
               buttonpic = IMG_Load("img/button_down.png");
               break;
          case 3:
               buttonpic = IMG_Load("img/button_left.png");
               break;
          case 4:
               buttonpic = IMG_Load("img/button_right.png");
               break;
      }

      buttonarea.x = (screen->w/2)-(buttonpic->w/2); //
      buttonarea.y = (screen->h/2)-(buttonpic->h/2); // Load Picture into the middle
      buttonarea.w = buttonpic->w;                   // of the screen ;)
      buttonarea.h = buttonpic->h;                   //

      SDL_BlitSurface(buttonpic, NULL, screen, &buttonarea);
      SDL_Flip(screen);
      SDL_FreeSurface(buttonpic);
}

void background(SDL_Surface *screen, int picture) {
     SDL_Surface *bmp;
     SDL_Rect    backgroundarea;

     switch(picture)
     {
       case 1:
           bmp = SDL_LoadBMP("img/background.bmp");
           break;
       case 2:
           bmp = SDL_LoadBMP("img/gameover.bmp");
           break;
       default:
           bmp = SDL_LoadBMP("img/error.bmp");
           break;
     }

     backgroundarea.x = 0;
     backgroundarea.y = 0;
     backgroundarea.w = bmp->w;
     backgroundarea.h = bmp->h;

     SDL_BlitSurface(bmp, NULL, screen, &backgroundarea);
     SDL_Flip(screen);
}

int main() {
     int random,button,counter,timer,exiting,ok,volume;
     int rantom;
     SDL_Event   joystick;
     SDL_Surface *screen;

     SDL_Surface *text_surface;
     SDL_Rect    textarea;
     SDL_Color   color={255,255,255}, bgcolor={0,0,0};
     TTF_Font    *font;
     char        str[255];

     /* generate a better rand */
     srand(time(NULL));

     /* Initialize SDL */
     SDL_Init(SDL_INIT_EVENTTHREAD | SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO);
     
     /* Initialize the screen */
     screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_DEPTH, SDL_SWSURFACE);

     /* Load Audio Device */
     Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 128);

     /* Load SDL_TTF Module to write */
     TTF_Init();
     font=TTF_OpenFont("fonts/FreeMonoBoldOblique.ttf",16);

     /* Remove Cursor */
     SDL_ShowCursor (0);

     /* Aktivate Joystick */
     SDL_JoystickOpen(0);

     counter = -1;
     timer = 1000;
     exiting = 0;
     rantom = 0;
     random = 0;
     volume = 64;

     play_music(1);

     while(1)
     {
        while(rantom == random)
        {
          random = rand()%4+1;
        }

        rantom = random;
        counter++;

        /* Draw Background Image */
        background(screen, 1);

        /* Draw Points on the screen */
        textarea.x = 10;
        textarea.w = 100;
        textarea.h = 16;
        textarea.y = 180;

        sprintf(str, " Points: %d ", counter);
        text_surface=TTF_RenderUTF8_Shaded(font,str,color,bgcolor);
        SDL_BlitSurface(text_surface,NULL,screen,&textarea);
        SDL_FreeSurface(text_surface);

        /* Show button, user should press */
        show_button(screen, random);

        while(button != random)
        {
          timer--;

          if(timer < 0) //if time is up
          {
            background(screen, 2); //draw gameover picture
            play_music(2); //play gameover music

            textarea.x = 100; //
            textarea.y = 112; // wrote the number of points into
            textarea.w = 100; // the _middle_ of the screen
            textarea.h = 16;  //

            sprintf(str, " Points: %d ", counter);
            text_surface=TTF_RenderUTF8_Shaded(font,str,color,bgcolor);
            SDL_BlitSurface(text_surface,NULL,screen,&textarea);
            SDL_Flip(screen);
            SDL_FreeSurface(text_surface);

            ok = 0;
            while(ok == 0)
            {
              while(SDL_PollEvent(&joystick))
              {
                switch(joystick.type)
                  {
                    case SDL_JOYBUTTONDOWN:
                        switch(joystick.jbutton.button)
                        {
                             case GP2X_BUTTON_START:  //Button START pressed -> Exit
                                  exiting = 1;        //say to exit
                                  ok = 1;             //leave loop
                                  break;
                             case GP2X_BUTTON_SELECT: //Other Button  pressed -> Reset
                                  play_music(1);      //play normal music
                                  timer = 1000;       //reset timer
                                  counter = -1;       //reset counter
                                  button = random;    //leave loop
                                  ok = 1;             //leave other loop
                                  break;
                        }
                  }
               }
            }
          }

          if(exiting == 1)
          {
            TTF_CloseFont(font);
            TTF_Quit();
            Mix_CloseAudio();
            SDL_Quit();
            chdir("/usr/gp2x");
            execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
            exit(0);
          }

          textarea.x = 10;
          textarea.w = 100;
          textarea.h = 16;
          textarea.y = 200;

          sprintf(str, " Time: %d ", timer);
          text_surface=TTF_RenderUTF8_Shaded(font,str,color,bgcolor);
          SDL_BlitSurface(text_surface,NULL,screen,&textarea);
          SDL_Flip(screen);
          SDL_FreeSurface(text_surface);

        while(SDL_PollEvent(&joystick))
        {
           switch(joystick.type)
           {
               case SDL_JOYBUTTONDOWN:
                   switch(joystick.jbutton.button)
                   {
                        case GP2X_BUTTON_START:
                             exiting = 1;
                             break;
                        case GP2X_BUTTON_UP:
                             button = 1;
                             break;
                        case GP2X_BUTTON_DOWN:
                             button = 2;
                             break;
                        case GP2X_BUTTON_LEFT:
                             button = 3;
                             break;
                        case GP2X_BUTTON_RIGHT:
                             button = 4;
                             break;
                        case GP2X_BUTTON_VOL_UP:
                             volume++;
                             Mix_VolumeMusic(volume);
                             play_music(1);
                             break;
                        case GP2X_BUTTON_VOL_DOWN:
                             volume--;
                             Mix_VolumeMusic(volume);
                             play_music(1);
                             break;
                   }
                   break;
               default:
                   
                   break;
            }
         }
      }
  }
}
